@@ -12,7 +12,7 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 #
-# RCS: @(#) $Id: safe.tcl,v 1.6.2.1 2001/04/03 22:54:38 hobbs Exp $
+# RCS: @(#) $Id: safe.tcl,v 1.16.4.2 2009/11/04 04:47:59 dgp Exp $
 
 #
 # The implementation is based on namespaces. These naming conventions
@@ -37,6 +37,10 @@ namespace eval ::safe {
     #
     ####
 
+    # Make sure that our temporary variable is local to this
+    # namespace.  [Bug 981733]
+    variable temp
+
     # Share the descriptions
     set temp [::tcl::OptKeyRegister {
 	{-accessPath -list {} "access path for the slave"}
@@ -73,7 +77,7 @@ namespace eval ::safe {
 	    upvar $v $v
 	}
 	set flag [::tcl::OptProcArgGiven -noStatics];
-	if {$flag && ($noStatics == $statics) 
+	if {$flag && (!$noStatics == !$statics) 
 	          && ([::tcl::OptProcArgGiven -statics])} {
 	    return -code error\
 		    "conflicting values given for -statics and -noStatics"
@@ -94,7 +98,7 @@ namespace eval ::safe {
 	set flag [::tcl::OptProcArgGiven -nestedLoadOk];
 	# note that the test here is the opposite of the "InterpStatics"
 	# one (it is not -noNested... because of the wanted default value)
-	if {$flag && ($nestedLoadOk != $nested) 
+	if {$flag && (!$nestedLoadOk != !$nested) 
 	          && ([::tcl::OptProcArgGiven -nested])} {
 	    return -code error\
 		    "conflicting values given for -nested and -nestedLoadOk"
@@ -294,7 +298,7 @@ namespace eval ::safe {
 	deletehook
     } {
 	# Create the slave.
-	if {[string compare "" $slave]} {
+	if {$slave ne ""} {
 	    ::interp create -safe $slave
 	} else {
 	    # empty argument: generate slave name
@@ -320,8 +324,8 @@ namespace eval ::safe {
 	    nestedok deletehook} {
 
 	# determine and store the access path if empty
-	if {[string equal "" $access_path]} {
-	    set access_path [uplevel #0 set auto_path]
+	if {$access_path eq ""} {
+	    set access_path [uplevel \#0 set auto_path]
 	    # Make sure that tcl_library is in auto_path
 	    # and at the first position (needed by setAccessPath)
 	    set where [lsearch -exact $access_path [info library]]
@@ -365,12 +369,44 @@ namespace eval ::safe {
 	    lappend slave_auto_path "\$[PathToken $i]"
 	    incr i
 	}
+	# Extend the access list with the paths used to look for Tcl
+	# Modules. We safe the virtual form separately as well, as
+	# syncing it with the slave has to be defered until the
+	# necessary commands are present for setup.
+
+	set morepaths [::tcl::tm::list]
+	while {[llength $morepaths]} {
+	    set addpaths $morepaths
+	    set morepaths {}
+
+	    foreach dir $addpaths {
+	    lappend access_path $dir
+	    Set [PathToken $i $slave] $dir
+	    lappend slave_auto_path "\$[PathToken $i]"
+	    lappend slave_tm_path   "\$[PathToken $i]"
+	    incr i
+
+		# [Bug 2854929]
+		# Recursively find deeper paths which may contain
+		# modules. Required to handle modules with names like
+		# 'platform::shell', which translate into
+		# 'platform/shell-X.tm', i.e arbitrarily deep
+		# subdirectories. The catch prevents complaints when
+		# no paths are added. Do nothing gracefully is 8.6+.
+
+		catch {
+		    lappend morepaths {*}[glob -nocomplain -directory $dir -type d *]
+	}
+	    }
+	}
+
+	Set [TmPathListName      $slave] $slave_tm_path
 	Set $nname $i
-	Set [PathListName $slave] $access_path
+	Set [PathListName        $slave] $access_path
 	Set [VirtualPathListName $slave] $slave_auto_path
 
-	Set [StaticsOkName $slave] $staticsok
-	Set [NestedOkName $slave] $nestedok
+	Set [StaticsOkName  $slave] $staticsok
+	Set [NestedOkName   $slave] $nestedok
 	Set [DeleteHookName $slave] $deletehook
 
 	SyncAccessPath $slave
@@ -435,7 +471,7 @@ proc ::safe::interpAddToAccessPath {slave path} {
 	# NB we need to add [namespace current], aliases are always
 	# absolute paths.
 	::interp alias $slave source {} [namespace current]::AliasSource $slave
-	::interp alias $slave load {} [namespace current]::AliasLoad $slave
+	::interp alias $slave load   {} [namespace current]::AliasLoad $slave
 
 	# This alias lets the slave use the encoding names, convertfrom,
 	# convertto, and system, but not "encoding system <name>" to set
@@ -444,6 +480,10 @@ proc ::safe::interpAddToAccessPath {slave path} {
 	::interp alias $slave encoding {} [namespace current]::AliasEncoding \
 		$slave
 
+	# Handling Tcl Modules, we need a restricted form of Glob.
+	::interp alias $slave glob {} [namespace current]::AliasGlob \
+		$slave
+
 	# This alias lets the slave have access to a subset of the 'file'
 	# command functionality.
 
@@ -459,27 +499,25 @@ proc ::safe::interpAddToAccessPath {slave path} {
 	# by Tcl_MakeSafe(3)
 
 
-	# Source init.tcl into the slave, to get auto_load and other
-	# procedures defined:
+	# Source init.tcl and tm.tcl into the slave, to get auto_load
+	# and other procedures defined:
 
-	# We don't try to use the -rsrc on the mac because it would get
-	# confusing if you would want to customize init.tcl
-	# for a given set of safe slaves, on all the platforms
-	# you just need to give a specific access_path and
-	# the mac should be no exception. As there is no
-	# obvious full "safe ressources" design nor implementation
-	# for the mac, safe interps there will just don't
-	# have that ability. (A specific app can still reenable
-	# that using custom aliases if they want to).
-	# It would also make the security analysis and the Safe Tcl security
-	# model platform dependant and thus more error prone.
-
-	if {[catch {::interp eval $slave\
+	if {[catch {::interp eval $slave \
 		{source [file join $tcl_library init.tcl]}} msg]} {
 	    Log $slave "can't source init.tcl ($msg)"
 	    error "can't source init.tcl into slave $slave ($msg)"
 	}
 
+	if {[catch {::interp eval $slave \
+		{source [file join $tcl_library tm.tcl]}} msg]} {
+	    Log $slave "can't source tm.tcl ($msg)"
+	    error "can't source tm.tcl into slave $slave ($msg)"
+	}
+
+	# Sync the paths used to search for Tcl modules. This can be
+	# done only now, after tm.tcl was loaded.
+	::interp eval $slave [list ::tcl::tm::add {*}[Set [TmPathListName $slave]]]
+
 	return $slave
     }
 
@@ -525,7 +563,7 @@ proc ::safe::interpDelete {slave} {
 		# remove the hook now, otherwise if the hook
 		# calls us somehow, we'll loop
 		Unset $hookname
-		if {[catch {eval $hook [list $slave]} err]} {
+		if {[catch {{*}$hook $slave} err]} {
 		    Log $slave "Delete hook error ($err)"
 		}
 	    }
@@ -601,7 +639,7 @@ proc ::safe::setLogCmd {args} {
     # if the slave argument is given, 
     # it will return the corresponding master global variable name
     proc PathToken {n {slave ""}} {
-	if {[string compare "" $slave]} {
+	if {$slave ne ""} {
 	    return "[InterpStateName $slave](access_path,$n)"
 	} else {
 	    # We need to have a ":" in the token string so
@@ -618,6 +656,10 @@ proc ::safe::setLogCmd {args} {
     proc VirtualPathListName {slave} {
 	return "[InterpStateName $slave](access_path_slave)"
     }
+    # returns the variable name of the complete tm path list
+    proc TmPathListName {slave} {
+	return "[InterpStateName $slave](tm_path_slave)"
+    }
     # returns the variable name of the number of items
     proc PathNumberName {slave} {
 	return "[InterpStateName $slave](access_path,n)"
@@ -636,15 +678,15 @@ proc ::safe::setLogCmd {args} {
     }
     # set/get values
     proc Set {args} {
-	eval Toplevel set $args
+	Toplevel set {*}$args
     }
     # lappend on toplevel vars
     proc Lappend {args} {
-	eval Toplevel lappend $args
+	Toplevel lappend {*}$args
     }
     # unset a var/token (currently just an global level eval)
     proc Unset {args} {
-	eval Toplevel unset $args
+	Toplevel unset {*}$args
     }
     # test existance 
     proc Exists {varname} {
@@ -673,7 +715,7 @@ proc ::safe::setLogCmd {args} {
     proc TranslatePath {slave path} {
 	# somehow strip the namespaces 'functionality' out (the danger
 	# is that we would strip valid macintosh "../" queries... :
-	if {[regexp {(::)|(\.\.)} $path]} {
+	if {[string match "*::*" $path] || [string match "*..*" $path]} {
 	    error "invalid characters in path $path"
 	}
 	set n [expr {[Set [PathNumberName $slave]]-1}]
@@ -691,28 +733,18 @@ proc ::safe::setLogCmd {args} {
     proc Log {slave msg {type ERROR}} {
 	variable Log
 	if {[info exists Log] && [llength $Log]} {
-	    eval $Log [list "$type for slave $slave : $msg"]
+	    {*}$Log "$type for slave $slave : $msg"
 	}
     }
 
-    
+
     # file name control (limit access to files/ressources that should be
     # a valid tcl source file)
     proc CheckFileName {slave file} {
-	# limit what can be sourced to .tcl
-	# and forbid files with more than 1 dot and
-	# longer than 14 chars
-	set ftail [file tail $file]
-	if {[string length $ftail]>14} {
-	    error "$ftail: filename too long"
-	}
-	if {[regexp {\..*\.} $ftail]} {
-	    error "$ftail: more than one dot is forbidden"
-	}
-	if {[string compare $ftail "tclIndex"] && \
-		[string compare -nocase [file extension $ftail]	".tcl"]} {
-	    error "$ftail: must be a *.tcl or tclIndex"
-	}
+	# This used to limit what can be sourced to ".tcl" and forbid files
+	# with more than 1 dot and longer than 14 chars, but I changed that
+	# for 8.4 as a safe interp has enough internal protection already
+	# to allow sourcing anything. - hobbs
 
 	if {![file exists $file]} {
 	    # don't tell the file path
@@ -725,21 +757,96 @@ proc ::safe::setLogCmd {args} {
 	}
     }
 
+    # AliasGlob is the target of the "glob" alias in safe interpreters.
+
+    proc AliasGlob {slave args} {
+	Log $slave "GLOB ! $args" NOTICE
+	set cmd {}
+	set at 0
+
+	set dir        {}
+	set virtualdir {}
+
+	while {$at < [llength $args]} {
+	    switch -glob -- [set opt [lindex $args $at]] {
+		-nocomplain -
+		-join       { lappend cmd $opt ; incr at }
+		-directory  {
+		    lappend cmd $opt ; incr at
+		    set virtualdir [lindex $args $at]
+
+		    # get the real path from the virtual one.
+		    if {[catch {set dir [TranslatePath $slave $virtualdir]} msg]} {
+			Log $slave $msg
+			return -code error "permission denied"
+		    }
+		    # check that the path is in the access path of that slave
+		    if {[catch {DirInAccessPath $slave $dir} msg]} {
+			Log $slave $msg
+			return -code error "permission denied"
+		    }
+		    lappend cmd $dir ; incr at
+		}
+		pkgIndex.tcl {
+		    # Oops, this is globbing a subdirectory in regular
+		    # package search. That is not wanted. Abort,
+		    # handler does catch already (because glob was not
+		    # defined before). See package.tcl, lines 484ff in
+		    # tclPkgUnknown.
+		    error "unknown command glob"
+		}
+		-* {
+		    Log $slave "Safe base rejecting glob option '$opt'"
+		    error      "Safe base rejecting glob option '$opt'"
+		}
+		default {
+		    lappend cmd $opt ; incr at
+		}
+	    }
+	}
+
+	Log $slave "GLOB = $cmd" NOTICE
+
+	if {[catch {::interp invokehidden $slave glob {*}$cmd} msg]} {
+	    Log $slave $msg
+	    return -code error "script error"
+	}
+
+	Log $slave "GLOB @ $msg" NOTICE
+
+	# Translate path back to what the slave should see.
+	set res {}
+	foreach p $msg {
+	    regsub -- ^$dir $p $virtualdir p
+	    lappend res $p
+	}
+
+	Log $slave "GLOB @ $res" NOTICE
+	return $res
+    }
 
     # AliasSource is the target of the "source" alias in safe interpreters.
 
     proc AliasSource {slave args} {
 
 	set argc [llength $args]
-	# Allow only "source filename"
-	# (and not mac specific -rsrc for instance - see comment in ::init
-	# for current rationale)
+	# Extended for handling of Tcl Modules to allow not only
+	# "source filename", but "source -encoding E filename" as
+	# well.
+	if {[lindex $args 0] eq "-encoding"} {
+	    incr argc -2
+	    set encoding [lrange $args 0 1]
+	    set at 2
+	} else {
+	    set at 0
+	    set encoding {}
+	}
 	if {$argc != 1} {
-	    set msg "wrong # args: should be \"source fileName\""
+	    set msg "wrong # args: should be \"source ?-encoding E? fileName\""
 	    Log $slave "$msg ($args)"
 	    return -code error $msg
 	}
-	set file [lindex $args 0]
+	set file [lindex $args $at]
 	
 	# get the real path from the virtual one.
 	if {[catch {set file [TranslatePath $slave $file]} msg]} {
@@ -760,7 +867,7 @@ proc ::safe::setLogCmd {args} {
 	}
 
 	# passed all the tests , lets source it:
-	if {[catch {::interp invokehidden $slave source $file} msg]} {
+	if {[catch {::interp invokehidden $slave source {*}$encoding $file} msg]} {
 	    Log $slave $msg
 	    return -code error "script error"
 	}
@@ -784,7 +891,7 @@ proc ::safe::setLogCmd {args} {
 	# Determine where to load. load use a relative interp path
 	# and {} means self, so we can directly and safely use passed arg.
 	set target [lindex $args 1]
-	if {[string length $target]} {
+	if {$target ne ""} {
 	    # we will try to load into a sub sub interp
 	    # check that we want to authorize that.
 	    if {![NestedOk $slave]} {
@@ -796,9 +903,9 @@ proc ::safe::setLogCmd {args} {
 	}
 
 	# Determine what kind of load is requested
-	if {[string length $file] == 0} {
+	if {$file eq ""} {
 	    # static package loading
-	    if {[string length $package] == 0} {
+	    if {$package eq ""} {
 		set msg "load error: empty filename and no package name"
 		Log $slave $msg
 		return -code error $msg
@@ -847,18 +954,45 @@ proc ::safe::setLogCmd {args} {
 	    error "\"$file\": is a directory"
 	}
 	set parent [file dirname $file]
-	if {[lsearch -exact $access_path $parent] == -1} {
+
+	# Normalize paths for comparison since lsearch knows nothing of
+	# potential pathname anomalies.
+	set norm_parent [file normalize $parent]
+	foreach path $access_path {
+	    lappend norm_access_path [file normalize $path]
+	}
+
+	if {[lsearch -exact $norm_access_path $norm_parent] == -1} {
 	    error "\"$file\": not in access_path"
 	}
     }
 
+    proc DirInAccessPath {slave dir} {
+	set access_path [GetAccessPath $slave]
+
+	if {[file isfile $dir]} {
+	    error "\"$dir\": is a file"
+	}
+
+	# Normalize paths for comparison since lsearch knows nothing of
+	# potential pathname anomalies.
+	set norm_dir [file normalize $dir]
+	foreach path $access_path {
+	    lappend norm_access_path [file normalize $path]
+	}
+
+	if {[lsearch -exact $norm_access_path $norm_dir] == -1} {
+	    error "\"$dir\": not in access_path"
+	}
+    }
+
     # This procedure enables access from a safe interpreter to only a subset of
     # the subcommands of a command:
 
     proc Subset {slave command okpat args} {
 	set subcommand [lindex $args 0]
 	if {[regexp $okpat $subcommand]} {
-	    return [eval {$command $subcommand} [lrange $args 1 end]]
+	    return [$command {*}$args]
 	}
 	set msg "not allowed to invoke subcommand $subcommand of $command"
 	Log $slave $msg
@@ -893,11 +1027,10 @@ proc ::safe::setLogCmd {args} {
 	set subcommand [lindex $args 0]
 
 	if {[regexp $okpat $subcommand]} {
-	    return [eval ::interp invokehidden $slave encoding $subcommand \
-		    [lrange $args 1 end]]
+	    return [::interp invokehidden $slave encoding {*}$args]
 	}
 
-	if {[string match $subcommand system]} {
+	if {[string first $subcommand system] == 0} {
 	    if {$argc == 1} {
 		# passed all the tests , lets source it:
 		if {[catch {::interp invokehidden \
